import os
import shutil
import logging
import tempfile
import threading
from normality import safe_filename

from servicelayer.archive.archive import Archive

log = logging.getLogger(__name__)


class VirtualArchive(Archive):

    def __init__(self, base_name):
        self.local = threading.local()
        self.base_name = base_name

    def _get_local_prefix(self, content_hash, temp_path=None):
        """Determine a temporary path for the file on the local file
        system."""
        if temp_path is None:
            if not hasattr(self.local, 'dir'):
                self.local.dir = tempfile.mkdtemp(prefix=self.base_name)
            temp_path = self.local.dir
        path_name = '%s.storagelayer' % content_hash
        return os.path.join(temp_path, path_name)

    def _local_path(self, content_hash, file_name, temp_path):
        path = self._get_local_prefix(content_hash, temp_path=temp_path)
        try:
            os.makedirs(path)
        except Exception:
            pass
        file_name = safe_filename(file_name, default='data')
        return os.path.join(path, file_name)

    def cleanup_file(self, content_hash, temp_path=None):
        """Delete the local cached version of the file."""
        if content_hash is None:
            return
        path = self._get_local_prefix(content_hash, temp_path=temp_path)
        try:
            shutil.rmtree(path, ignore_errors=True)
        except Exception:
            pass



from fakeredis import FakeRedis
from redis import ConnectionPool, Redis

from servicelayer import settings


def get_fakeredis():
    if not hasattr(settings, '_redis_fake'):
        settings._redis_fake = FakeRedis(decode_responses=True)
    return settings._redis_fake


def get_redis_pool():
    if not hasattr(settings, '_redis_pool'):
        settings._redis_pool = ConnectionPool.from_url(settings.REDIS_URL,
                                                       decode_responses=True)
    return settings._redis_pool


def get_redis():
    """Create a redis connection."""
    if settings.REDIS_URL is None:
        return get_fakeredis()
    return Redis(connection_pool=get_redis_pool(), decode_responses=True)


def make_key(*criteria):
    """Make a string key out of many criteria."""
    criteria = [c or '' for c in criteria]
    criteria = [str(c) for c in criteria]
    return ':'.join(criteria)



from pkg_resources import iter_entry_points

EXTENSIONS = {}


def get_entry_points(section):
    """Load all Python classes registered at a given entry point."""
    if section not in EXTENSIONS:
        EXTENSIONS[section] = {}
    if not EXTENSIONS[section]:
        for ep in iter_entry_points(section):
            EXTENSIONS[section][ep.name] = ep.load()
    return EXTENSIONS[section]


def get_entry_point(section, name):
    return get_entry_points(section).get(name)


def get_extensions(section):
    """Iterate entry point objects."""
    return list(get_entry_points(section).values())



import sys
from hashlib import sha1

BUF_SIZE = 1024 * 1024 * 16


def checksum(file_name):
    """Generate a hash for a given file name."""
    digest = sha1()
    with open(file_name, 'rb') as fh:
        while True:
            block = fh.read(BUF_SIZE)
            if not block:
                break
            digest.update(block)
    return str(digest.hexdigest())


def decode_path(file_path):
    """Decode a path specification into unicode."""
    if file_path is None:
        return
    if isinstance(file_path, bytes):
        file_path = file_path.decode(sys.getfilesystemencoding())
    return file_path
